import collections


class OrderedClassMembers(type):
    @classmethod
    def __prepare__(self, name, bases):
        return collections.OrderedDict()

    def __new__(self, name, bases, classdict):
        classdict['__ordered__'] = [key for key in classdict.keys() if  # __classcell__ passed to __new__
                                    key not in ('__module__', '__qualname__', '__classcell__')]

        for each in bases:  # add keys from base classes also
            odict = getattr(each, '__ordered__', each.__dict__)

            classdict['__ordered__'] = [key for key in odict if key not in ('__module__', '__qualname__')] + \
                                       classdict['__ordered__']

        return type.__new__(self, name, bases, classdict)



class ManagementCommandNotFound(Exception):
    pass


def execute_from_command_line(args):  # sys args
    command_name = args[1]
    command_args = args[2:]
    from trellio.management.commands.base import ManagementRegistry
    if not ManagementRegistry.get(command_name):
        raise ManagementCommandNotFound
    command_class = ManagementRegistry.get(command_name)
    command_class(command_args).run()



import logging
import logging.handlers
import smtplib

logger = logging.getLogger(__name__)


class BufferingSMTPHandler(logging.handlers.BufferingHandler):
    def __init__(self, mailhost, mailport, fromaddr, toaddrs, subject, capacity, password):
        logging.handlers.BufferingHandler.__init__(self, capacity)
        self.mailhost = mailhost
        self.mailport = mailport
        self.fromaddr = fromaddr
        self.subject = subject
        self.toaddrs = toaddrs
        self.setFormatter(logging.Formatter("%(asctime)s %(levelname)-5s %(message)s"))
        self.smtp = smtplib.SMTP(self.mailhost, self.mailport)
        self.smtp.ehlo()
        self.smtp.starttls()
        self.smtp.login(fromaddr, password)

    def handleError(self, record):
        print(record)  # overriding original and not doing anything

    def flush(self):
        if len(self.buffer) > 0:
            try:
                msg = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" % (
                    self.fromaddr, ",".join(self.toaddrs), self.subject)
                for record in self.buffer:
                    s = self.format(record)
                    msg = msg + s + "\r\n"
                self.smtp.sendmail(self.fromaddr, self.toaddrs, msg)
            except Exception as e:
                self.handleError(e)  # no particular record
            self.buffer = []



from trellio.views import HTTPView,TCPView
from trellio.services import api,get
from ..clients import ServiceBCLient


class TCPView1(TCPView):

    @api
    async def servicea_tcp_api1(self,params):
        return {'msg':'Success'}


class HTTPView1(HTTPView):

    @get(path='/serviceA/1/')
    async def enpoint_one(self,request):
        return await ServiceBCLient().serviceb_tcp_api1(params={})


